package elokuvakatalogi;

import java.io.File;
import java.io.PrintStream;
import java.util.List;

/** Huolehtii Elokuvat ja Nayttelijat -luokkien välisestä yhteistyöstä
 * Lukee ja kirjoittaa elokuvakatalogin tiedostoon pyytämällä apua
 * avustajaluokiltaan 
 * @author Tuukka
 * @version 28.2.2025
 *
 */
public class Elokuvakatalogi {
    
    private Elokuvat elokuvat = new Elokuvat();
    private Nayttelijat nayttelijat = new Nayttelijat();
    
    /**
     * Luetaan elokuvien ja näyttelijöiden tiedot
     * @param hakemisto mistä tiedostoja luetaan
     * @throws SailoException jos tiedoston lukeminen ei onnistu
     */
    public void lueTiedostosta(String hakemisto) throws SailoException {
        File dir = new File(hakemisto);
        dir.mkdir();
        try {
            elokuvat = new Elokuvat();
            nayttelijat = new Nayttelijat();
            elokuvat.lueTiedostosta(hakemisto);
            nayttelijat.lueTiedostosta(hakemisto);
        } catch (SailoException ex) {
            throw new SailoException(hakemisto + "-kansiosta ei onnistuttu lukemaan tiedostoja");
        }
    }

    
    /**
     * Lisää elokuvan katalogiin käyttäen apunaan Elokuvat-luokkaa
     * @param elokuva Elokuva, joka halutaan lisätä elokuvakatalogiin
     * @throws SailoException mikäli tietokanta on täynnä
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Elokuvakatalogi katalogi = new Elokuvakatalogi();
     * Elokuva elokuva1 = new Elokuva();
     * katalogi.lisaa(elokuva1); katalogi.lisaa(elokuva1); 
     * katalogi.lisaa(elokuva1); katalogi.lisaa(elokuva1); katalogi.lisaa(elokuva1); 
     * 
     * </pre>
     */
    public void lisaa(Elokuva elokuva) throws SailoException {
        elokuvat.lisaa(elokuva);
    }
    
    /**
     * Palauttaa elokuvien lukumäärän käyttäen apunaan 
     * Elokuvat-luokkaa
     * @return Elokuvien lukumäärä
     * Elokuvakatalogi katalogi = new Elokuvakatalogi();
     * katalogi.getElokuvia() === 0;
     * Elokuva elo1 = new Elokuva();
     * katalogi.lisaa(elo1);
     * katalogi.getElokuvia() === 1;
     */
    public int getElokuvia() {
        return elokuvat.getLkm();
    }
    
    /**
     * Palautetaan elokuva, joka on paikassa i käyttäen Elokuvat-luokkaa
     * @param i Indeksi, josta etsitään elokuvaa
     * @return Elokuva indeksissä i
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Elokuvakatalogi katalogi = new Elokuvakatalogi();
     * Elokuva elo1 = new Elokuva(); Elokuva elo2 = new Elokuva();
     * katalogi.lisaa(elo1); katalogi.lisaa(elo2);
     * katalogi.annaElokuva(0) === elo1;
     * katalogi.annaElokuva(1) === elo2;
     * </pre>
     */
    public Elokuva annaElokuva(int i) {
        return elokuvat.anna(i);
    }
    
    /**
     * Lisää näyttelijän katalogiin käyttäen apunaan Nayttelijat-luokkaa
     * @param nayttelija joka halutaan lisätä katalogiin
     */
    public void lisaa(Nayttelija nayttelija) {
        nayttelijat.lisaa(nayttelija);
    }
    
    /**
     * @return Näyttelijöiden lukumäärä
     */
    public int getNayttelijoita() {
        return nayttelijat.getLkm();
    }
    
    /**
     * Palauttaa tietyn elokuvan näyttelijät
     * @param elokuva jonka näyttelijät halutaan
     * @return kyseisen elokuvan näyttelijät
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Elokuvakatalogi katalogi = new Elokuvakatalogi();
     * Elokuva elo1 = new Elokuva();
     * Elokuva elo2 = new Elokuva();
     * Elokuva elo3 = new Elokuva();
     * elo1.rekisteroi(); elo2.rekisteroi(); elo3.rekisteroi();
     * int id1 = elo1.getId(); int id2 = elo2.getId();
     * Nayttelija nayt1 = new Nayttelija(id1); 
     * Nayttelija nayt2 = new Nayttelija(id1);
     * Nayttelija nayt3 = new Nayttelija(id2);
     * katalogi.lisaa(nayt1); katalogi.lisaa(nayt2); katalogi.lisaa(nayt3);
     * katalogi.annaNayttelijat(elo1).size() === 2;
     * katalogi.annaNayttelijat(elo2).size() === 1;
     * katalogi.annaNayttelijat(elo3).size() === 0;
     * </pre>
     */
    public List<Nayttelija> annaNayttelijat(Elokuva elokuva) {
       return nayttelijat.annaNayttelijat(elokuva.getId());
    }
    
    /**
     * Tulostaa elokuvan tiedot
     * @param os minne tiedot tulostetaan
     * @param elokuva jonka tiedot tulostetaan
     */
    public void tulosta(PrintStream os, Elokuva elokuva) {
        os.println("======================");
        elokuva.tulosta(os);
        os.println("======================");
        List<Nayttelija> nayttelijaLista = annaNayttelijat(elokuva);
        for(Nayttelija nayt : nayttelijaLista) {
            nayt.tulosta(os);
        }
    }
    
    /**
     * Tallennetaan elokuvakatalogin tiedot käyttäen hyväksi Elokuvat-luokkaa
     * @param hakemisto mihin hakemistoon elokuvat ja näyttelijät tallennetaan
     * @throws SailoException jos tallennus ei onnistu
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * #import java.io.File;
     * #import java.util.List;
     * String hakemisto = "testikatalogi";
     * Elokuvakatalogi katalogi = new Elokuvakatalogi();
     * Elokuva elo = new Elokuva(); elo.rekisteroi(); elo.taytaSaw(); katalogi.lisaa(elo);
     * Nayttelija nayt = new Nayttelija(); nayt.taytaWillemDafoe(1); katalogi.lisaa(nayt);
     * katalogi.tallenna(hakemisto);
     * Elokuvakatalogi katalogi2 = new Elokuvakatalogi();
     * katalogi2.getElokuvia() === 0; katalogi2.getNayttelijoita() === 0;
     * katalogi2.lueTiedostosta(hakemisto);
     * katalogi2.getElokuvia() === 1; katalogi2.getNayttelijoita() === 1;
     * elo.getNimi() === katalogi2.annaElokuva(0).getNimi();
     * List<Nayttelija> nayttelijat = katalogi2.annaNayttelijat(elo);
     * nayt.getNimi() === nayttelijat.get(0).getNimi();
     * </pre>
     */
    public void tallenna(String hakemisto) throws SailoException {
        try {
            elokuvat.tallenna(hakemisto);
            nayttelijat.tallenna(hakemisto);
        } catch (SailoException ex) {
            throw new SailoException("Tallennus ei onnistunut kansioon" + hakemisto);
        }
    }

}
