package elokuvakatalogi;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Scanner;

/** - Pitää yllä rekisteriä elokuvista eli lisää ja poistaa elokuvia
 * katalogista
 * - Lukee ja kirjoittaa elokuvakatalogin tiedostoo
 * - Etsii ja lajittelee elokuvia erilaisten kriteerien perusteella
 * @author Tuukka
 * @version 27.2.2025
 *
 */
public class Elokuvat {
    
    private static final int MAX_ELOKUVIA = 10;
    private static final int KASVATA_TAULUKKOA = 10;
    private int lkm = 0;
    private Elokuva[] alkiot;
    
    /**
     * Alustava taulukko elokuville
     */
    public Elokuvat() {
        this.alkiot = new Elokuva[MAX_ELOKUVIA];
    }
    
    /** Lisää elokuvan rekisteriin
     * @param elokuva Elokuva, joka lisätään rekisteriin
     * @throws SailoException mikäli tietokanta on täynnä
     */
    public void lisaa(Elokuva elokuva) throws SailoException {
        if(lkm >= alkiot.length) kasvataElokuvaTaulukkoa();
        this.alkiot[this.lkm] = elokuva;
        this.lkm++;
    }
    
    /**
     * Kasvattaa elokuvataulukkoa valitun vakion verran, kun taulukko on täynnä
     */
    private void kasvataElokuvaTaulukkoa() {
        Elokuva[] uudetAlkiot = new Elokuva[lkm+KASVATA_TAULUKKOA]; 
        for (int i = 0; i<lkm; i++) {
            uudetAlkiot[i] = alkiot[i];
        }
        alkiot = uudetAlkiot;
    }

    /**
     * @return Elokuvien lukumäärän
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Elokuvat elokuvat = new Elokuvat();
     * elokuvat.getLkm() === 0;
     * Elokuva elokuva1 = new Elokuva(); Elokuva elokuva2 = new Elokuva();
     * elokuvat.lisaa(elokuva1); elokuvat.lisaa(elokuva2);
     * elokuvat.getLkm() === 2;
     * elokuvat.lisaa(elokuva1); elokuvat.lisaa(elokuva2); elokuvat.lisaa(elokuva1);
     * elokuvat.lisaa(elokuva2);
     * </pre>
     */
    public int getLkm() {
        return this.lkm;
    }
    
    /** Palauttaa elokuvan, joka on paikassa i
     * @param i Indeksi, josta elokuvaa etsitään
     * @return Elokuva paikalla i
     */
    public Elokuva anna(int i) {
        if (i< 0 || this.lkm <= i) {
            throw new IndexOutOfBoundsException("Laiton indeksi " + i);
        }
        return alkiot[i];
    }
    
    /**
     * Tallennetaan elokuvien tiedot halutun hakemiston tiedostoon
     * @param hakemisto mihin tiedot tallennetaan
     * @throws SailoException mikäli tallennus ei onnistu
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * #import java.io.File;
     * String hakemisto = "testielokuvat";
     * String tiedNimi = hakemisto+"/leffat";
     * File ftied = new File(tiedNimi+".dat");
     * File dir = new File(hakemisto);
     * dir.mkdir();
     * ftied.delete();
     * Elokuvat elokuvat = new Elokuvat();
     * Elokuva elo1 = new Elokuva(); elo1.taytaSaw(); elokuvat.lisaa(elo1);
     * Elokuva elo2 = new Elokuva(); elo2.taytaSaw(); elokuvat.lisaa(elo2);
     * elokuvat.tallenna(hakemisto);
     * Elokuvat elokuvat2 = new Elokuvat();
     * elokuvat2.getLkm() === 0;
     * elokuvat2.lueTiedostosta(hakemisto);
     * elokuvat2.getLkm() === 2;
     * elo1.getNimi() === elokuvat2.anna(0).getNimi();
     * elo2.getNimi() === elokuvat2.anna(1).getNimi();
     * </pre>
     */
    public void tallenna(String hakemisto) throws SailoException {
        File talTied = new File(hakemisto + "/leffat.dat");
        try (PrintStream fo = new PrintStream(new FileOutputStream(talTied,false))) {
           for(int i = 0; i<this.getLkm(); i++) {
               Elokuva elokuva = this.anna(i);
               fo.println(elokuva.toString());
           }
        } catch (FileNotFoundException ex) {
            throw new SailoException("Tiedosto " + talTied.getName() + " ei aukea");
        }
    }
    
    /**
     * Lukee elokuvien tiedot tiedostosta
     * @param hakemisto mistä luettava tiedosto etsitään
     * @throws SailoException jos luettavaa tiedostoa ei löydy
     */
    public void lueTiedostosta(String hakemisto) throws SailoException {
        File tiedLuet = new File(hakemisto + "/leffat.dat");
        
        try (Scanner fi = new Scanner(new FileInputStream(tiedLuet))) {
            while (fi.hasNext()) {
                String elokuvaTiedot = fi.nextLine();
                if (elokuvaTiedot.isBlank()) continue;
                Elokuva elokuva = new Elokuva();
                elokuva.parse(elokuvaTiedot);
                lisaa(elokuva);
            }
        } catch (FileNotFoundException ex) {
            throw new SailoException("Ei pystytä lukee tiedostoa" + tiedLuet.getName());
        }
    }
}
