package elokuvakatalogi;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Scanner;

/**
 * - Pitää yllä rekisteriä näyttelijöistä eli osaa
 *  lisätä ja poistaa näyttelijän
 * - Lukee ja kirjoittaa näyttelijät tiedostoon
 * - Osaa etsiä ja lajitella
 * @author Tuukka
 * @version 17.3.2025
 *
 */
public class Nayttelijat {
    
    private final Collection<Nayttelija> nayttelijat = new ArrayList<Nayttelija>();
    
    /**
     * Alustava lista näyttelijöille
     */
    public Nayttelijat() {
        //
    }
    
    /**
     * @param nayttelija joka halutaan lisätä näyttelijälistaan
     */
    public void lisaa(Nayttelija nayttelija) {
        nayttelijat.add(nayttelija);
    }
    
    /**
     * @return näyttelijöiden lukumäärän
     */
    public int getLkm() {
        return nayttelijat.size();
    }
    
    /**
     * Etsii ja palauttaa halutun elokuvan näyttelijät
     * @param id elokuvan id
     * @return elokuvan näyttelijät
     * @example
     * <pre name="test">
     * Nayttelijat nayttelijat = new Nayttelijat();
     * Nayttelija nayt1 = new Nayttelija(1); 
     * Nayttelija nayt2 = new Nayttelija(1);
     * Nayttelija nayt3 = new Nayttelija(1); 
     * Nayttelija nayt4 = new Nayttelija(2);
     * nayttelijat.lisaa(nayt1); nayttelijat.lisaa(nayt2);
     * nayttelijat.lisaa(nayt3); nayttelijat.lisaa(nayt4);
     * nayttelijat.annaNayttelijat(1).size() === 3;
     * nayttelijat.annaNayttelijat(2).size() === 1;
     * nayttelijat.annaNayttelijat(3).size() === 0;
     * </pre>
     */
    public List<Nayttelija> annaNayttelijat(int id) {
        List<Nayttelija> palautetut = new ArrayList<Nayttelija>();
        for(Nayttelija nay : nayttelijat) {
            if (nay.getId()==id) palautetut.add(nay);
        }
        return palautetut;
    }
    
    /**
     * Lukee elokuvien tiedot tiedostosta
     * @param hakemisto mistä luettava tiedosto etsitään
     * @throws SailoException jos luettavaa tiedostoa ei löydy
     */
    public void lueTiedostosta(String hakemisto) throws SailoException {
        File tiedLuet = new File(hakemisto + "/nayttelijat.dat");
        
        try (Scanner fi = new Scanner(new FileInputStream(tiedLuet))) {
            while (fi.hasNext()) {
                String nayttelijaTiedot = fi.nextLine();
                if (nayttelijaTiedot.isBlank()) continue;
                Nayttelija nayt = new Nayttelija();
                nayt.parse(nayttelijaTiedot);
                lisaa(nayt);
            }
        } catch (FileNotFoundException ex) {
            throw new SailoException("Ei pystytä lukee tiedostoa" + tiedLuet.getName());
        }
    }

    /**
     * Tallennetaan elokuvien tiedot halutun hakemiston tiedostoon
     * @param hakemisto mihin tiedot tallennetaan
     * @throws SailoException mikäli tallennus ei onnistu
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * #import java.io.File;
     * #import java.util.List;
     * String hakemisto = "testielokuvat";
     * String tiedNimi = hakemisto+"/nayttelijat";
     * File ftied = new File(tiedNimi+".dat");
     * File dir = new File(hakemisto);
     * dir.mkdir();
     * ftied.delete();
     * Nayttelijat nayttelijat = new Nayttelijat();
     * Nayttelija nayt1 = new Nayttelija(); nayt1.taytaWillemDafoe(1); nayttelijat.lisaa(nayt1);
     * Nayttelija nayt2 = new Nayttelija(); nayt2.taytaWillemDafoe(2); nayttelijat.lisaa(nayt2);
     * nayttelijat.tallenna(hakemisto);
     * Nayttelijat nayttelijat2 = new Nayttelijat();
     * nayttelijat2.getLkm() === 0;
     * nayttelijat2.lueTiedostosta(hakemisto);
     * nayttelijat2.getLkm() === 2;
     * List<Nayttelija> nayttelijaLista1 = nayttelijat2.annaNayttelijat(1);
     * List<Nayttelija> nayttelijaLista2 = nayttelijat2.annaNayttelijat(2);
     * nayt1.getNimi() === nayttelijaLista1.get(0).getNimi();
     * nayt2.getNimi() === nayttelijaLista2.get(0).getNimi();
     * </pre>
     */
    public void tallenna(String hakemisto) throws SailoException {
        File talTied = new File(hakemisto + "/nayttelijat.dat");
        try (PrintStream fo = new PrintStream(new FileOutputStream(talTied,false))) {
           for(Nayttelija nayt : nayttelijat) {
               fo.println(nayt.toString());
           }
        } catch (FileNotFoundException ex) {
            throw new SailoException("Tiedosto " + talTied.getName() + " ei aukea");
        }
    }

}
